import edu.wsu.KheperaSimulator.*;
import java.util.ArrayList;
public class Channel implements Runnable{
	
	public static int NUMBER_OF_SENSORS = 8; // number of RobotController sensors
	public static int STORE_COUNT = 5; // Number of sensor input values to be stored
	
	private RobotController rc;
	private ArrayList<Integer>[] lightValues;
	private ArrayList<Integer>[] distValues;
	boolean finished;
	
	/**
	 * Create a new Channel, store sensor values in lists.
	 * 
	 * @param rc The RobotController used for input values
	 */
	public Channel(RobotController rc){
		this.rc = rc;
		lightValues = new ArrayList[NUMBER_OF_SENSORS];
		distValues = new ArrayList[NUMBER_OF_SENSORS];
		for(int i=0;i<NUMBER_OF_SENSORS;i++){
			lightValues[i] = new ArrayList<Integer>(); // One list pr. sensor.
			distValues[i] = new ArrayList<Integer>();
		}
		finished = false;
	}
	
	/**
	 * Gets the validated mean of 5 sensor values
	 * 
	 * @param id SensorID
	 * @return median value of the stored light values
	 */
	public int getLightValue(int id){
		return getLightMeanValue(id);
	}
	
	public int getDistanceValue(int id){
		/* not implemented */
		return getDistanceMeanValue(id);
	}
	
	/**
	 * run() - the main loop of the Channel class. Update the list of all sensors.
	 */
	public void run(){
		while(!finished){
			for(int id=0;id<NUMBER_OF_SENSORS;id++){
				updateLightValue(id); // update all light sensors
				updateDistValue(id);
			}
			try{
				Thread.sleep(5);
			}catch (InterruptedException e){}
			
		}
	}
	
	/**
	 * Updates light value of a sensor and validates the input.
	 * 
	 * @param id sensorID
	 */
	private synchronized void updateLightValue(int id){
		if(id >= 0 && id < NUMBER_OF_SENSORS){
			int value = rc.getLightValue(id); // get the input value from the RobotController
			if( value >= 0 && value <= 500){ // validate the input
				lightValues[id].add(0, new Integer(value));
				/* Keep the list the size of STORE_COUNT */
				while(lightValues[id].size() > STORE_COUNT){
					lightValues[id].remove(STORE_COUNT);
				}
			}
		}
	}
	

	private synchronized void updateDistValue(int id) 
	{
		if (id >= 0 && id < NUMBER_OF_SENSORS) 
		{
			int value = rc.getDistanceValue(id);
			if(value >= 0 && value <= 1023) {
				distValues[id].add(0, new Integer(value));
				while(distValues[id].size() > STORE_COUNT) 
					distValues[id].remove(STORE_COUNT);
			}
		}
	}

	/**
	 * Get the mean value of the stored values of a light sensor.
	 * 
	 * @param id sensorID
	 * @return mean value
	 */
	private synchronized int getLightMeanValue(int id){
		int value = 0;
		int count = 0;
		for(int i=0;i<lightValues[id].size();i++){
			if (lightValues[id].get(i) != null) 
			{
				value += ((Integer) lightValues[id].get(i)).intValue();
				count++;
			}
		}
		if(count > 0){
			return value/count; 
		}else{
			return 0;
		}
	}
	
	private synchronized int getDistanceMeanValue(int id) 
	{
		int value = 0; 
		int count = 0;
		for(int i = 0; i < distValues[id].size(); ++i) {
			if (distValues[id].get(i) != null) 
			{
				value += ((Integer) distValues[id].get(i)).intValue();
				++ count;
			}
		}
		if (count > 0) 
			return value/count;
		else
			return 0;
	}
	
	/**
	 * Stop the Channel processing.
	 *
	 */
	public void finish(){
		finished = true;
	}
}
